(*bote  outils de graphes, belle version*)

let diese = -1;;

(* dfinition du type graphe - constructeurs et destructeurs *)

type graphe = {matrice : int array array ;
	       nb_n : int ;
	       nb_d : int ;};;

let get_matrice graph = 
  graph.matrice;;

let get_n graph = 
  graph.nb_n;;

let get_d graph =
  graph.nb_d;;

let make_graph mat n d = {matrice = mat ;  nb_n = n ; nb_d = d}

(*Outils de graphes, tableau des voisins d'un sommet et vecteur_h*)

let voisins graphe sommet = (*liste des voisins d'un sommet*)
  if (sommet>(get_n graphe) -1) then failwith "ce sommet n'est pas dans le graphe"
  else (get_matrice graphe).(sommet);;


exception Trouve of int;;
exception PasTrouve;;

exception Oui;;
exception Non;;

let est_voisin_aux graphe s1 s2 =
  let voisins_de_s1 = voisins graphe s1 in
    for i=0 to (get_d graphe)-1 do
      if ((voisins_de_s1).(i) = s2) then raise Oui;
    done;
    raise Non;;

let est_voisin graphe s1 s2 = (*teste si s1 et s2 sont voisins*)
  try (est_voisin_aux graphe s1 s2)
  with
      Oui -> true
    | Non -> false;;

let position graphe s1 s2 = (*quelle est la position de s2 dans les voisins de s1?*)
  try (
    let voisins_de_s1 = voisins graphe s1 and compt = ref 0 in
      for i=0 to (get_d graphe)-1 do
	begin
	  if ((voisins_de_s1).(i) = s2) then raise (Trouve (!compt));
	  compt := !compt + 1;
	end;
      done;
      raise PasTrouve;
  )
  with
      Trouve(i)-> i
    | PasTrouve -> failwith "position : les 2 sommets ne sont pas voisins!";;

let i_eme_voisin graphe s1 i = (*quel est le i-ime voisin de s1 ?*)
  if ( (i<0)|| (i>= get_d graphe) ) 
  then failwith "i_eme_voisin : numero de voisin trop grand"
  else
    let voisins_de_s1 = voisins graphe s1 in
      voisins_de_s1.(i);;

let vecteur_h graphe sommet  = (* le vecteur de voisinage H(sommet)*)
  let tab = Array.make (get_d graphe) 0 in
  let voisins_sommets = voisins graphe sommet in
    for i=0 to ((get_d graphe) -1) do
      begin
	let pos = (position graphe (i_eme_voisin graphe sommet i)  sommet) in
	  tab.(i) <- pos ;
      end;
    done;
    tab;;


(*outils de traduction*)
let i2s int = (*entier en une chaine*)
  if int = -1 then "#"
  else string_of_int int;;
let i2c int = (*entier en char : code ascii*)
  char_of_int (int);;
let c2i c = (*chaine en entier : code ascii*)
  int_of_char c;;

let line2string graphe i = (*transforme la ligne i en une chaine avec space+\n  la fin*)
  let d = get_d graphe and matrice = get_matrice graphe and str=ref "" in
    for j=0 to (d-1) do
      begin
	str:=!str^(i2s matrice.(i).(j))^" ";
      end
    done;
    str:=!str^"\n";
    !str;;

(*fonctions principales du module : graph2file et file2graph*)

let graph2file graphe file = (*transforme le graphe en son fichier cod*) 
  let out_channel = open_out file in
    output_string out_channel (i2s(get_n graphe)^" "^i2s(get_d graphe)^"\n");
    for i=0 to ((get_n graphe)-1) do 
    output_string out_channel (line2string graphe i)
    done;
    close_out out_channel;;

(*nouvelle fonction du 27/3/01*)
exception End_of_String ;;

let next_number string =
  let res = ref 0 and len = ref (String.length string)
		       and i = ref 0 
		       and beginning = ref 0
		       and lg = ref 0 in
    if !len = 0 then raise End_of_String
    else let c= ref string.[0] in
      begin
	while (!c = ' ') do
	  begin
	    i := !i + 1;
	    if i=len then raise End_of_String
	    else c := string.[!i];
	  end;
	done;
	if (!c = '#') then 
	  begin 
	    res := diese;
	    i := !i + 1;
	  end
	else 
	  begin
	    beginning := !i;
	    while (!c <> ' ' && !i< !len) do
	      begin
		i := !i + 1;
		if (!i <> !len) then c := string.[!i]
	      end
	    done;
	    lg:=  !i - !(beginning) ;
	    res  := int_of_string(String.sub string !(beginning) !(lg));
	  end;
	  (!res, String.sub string !i (!len - !i))
      end;;

(*rcupre n nombres dans "   51515 24 7 09     "*)
let extract_numbers string n =
  let tmp = ref (0,string) 
  and res = ref 0 
  and tab_res = Array.make n 0 in
    for i=0 to n-1 do
      begin
	tmp := next_number (snd (!tmp));
	res := fst !tmp;
	tab_res.(i) <- !res;
      end;
    done;
    tab_res;;

let make_mat n d va= (*construction d'une matrice dont les cases sont diffrentes*)
  let v=Array.make n (Array.make d va) in
    for k=1 to (n-1) do v.(k)<- Array.make d va done;
    v;;

(* let file2graph file = (*transformation d'un fichier cod en son graphe*)
  let in_channel = open_in file in
  let fst = input_line in_channel in
  let (n,d) = recup_nbs fst in
  let  mat_res = make_mat n d 0 
  in
    for i=0 to (n-1) do
      begin
	let chaine =  input_line in_channel in
	  (* print_string (chaine);*)
	  if ((String.length (chaine)) <> 2*d)
	  then failwith ("file2graph : pb avec le codage de la ligne "^(i2s i)^" de la matrice")
	  else
	    for j=0 to (d-1) do
	      mat_res.(i).(j) <- (c2i(chaine.[2*j])-48)
	    done
      end
    done;
  make_graph mat_res n d ;;	*)

(*27/3 aussi*)
(*
let next_non_empty line in_channel =
  try (
    let tmp = input_line in_channel in
      while ...

A FAIRE ...

  )
  with
      End_of_file -> failwith("pb de codage du fichier")
*)

let file2graph file = (*transformation d'un fichier cod en son graphe*)
  let in_channel = open_in file in
  let fst = input_line in_channel in				    
  let recup_nb = extract_numbers fst 2 in
  let n = recup_nb.(0) and d = recup_nb.(1) in
  let  mat_res = make_mat n d 0 
  in
    for i=0 to (n-1) do
      begin
	let string = input_line in_channel in
	  try 
	    (
	      let res = extract_numbers string d in
		for j = 0 to (d-1) do
		  begin
		    mat_res.(i).(j) <- res.(j);
		  end;
		done;
	    )
	  with
	      End_of_String -> failwith ("pb avec la ligne "^string_of_int(i+1)^" du fichier");
      end;
    done;
    make_graph mat_res n d ;;	    


(* les automates qui sont aux noeuds et leur transition*)
type automate = {nb_a_d : int ; delta : int array -> int};;
let get_a_d auto = auto.nb_a_d;;
let get_a_delta auto = auto.delta;;
let make_automate d delt = {nb_a_d = d; delta = delt};;

let del_essai (tab: int array) = 
  let compt = ref 0 in
    for i=0 to ((Array.length tab)-1) do
      compt := !compt + tab.(i);
    done;
    !compt;;

let auto_essai = {nb_a_d = 2 ; delta = del_essai};;

(* --- applique la fonction de transition  un automate en fonction de ses voisins --- *)

let apply_fct graph tab auto = (* applique la fonction de transition de l'automate aux tats des sommets de graph contenus dans tab*)
  let n = get_n graph and d = get_d graph and mat = get_matrice graph in
  let res = Array.make n 0 in
    for i=0 to (n-1) do
      let tab_etats_vois = Array.make d 0 and res_auto = Array.make d 0 in
	for j=0 to (d-1) do
	    tab_etats_vois.(j) <- (if (mat.(i).(j) <> diese) then tab.(mat.(i).(j)) else diese)
	done;
	res.(i) <- (get_a_delta auto) tab_etats_vois 
    done;
    res ;;


let apply_fct_iter_and_treat graph tab auto nb_times treatment =
  let compt = ref 0 in 
    while (!compt < nb_times) do
      begin
	treatment (apply_fct graph tab auto);
	compt := !compt+1;
      end;
    done;;




